package com.xhkjedu.sexam.controller.exam;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.sexam.model.exam.EBase;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.service.exam.EBaseService;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @classname EBaseController
 * @description 考试基础信息
 * @date 2022/7/18 9:26
 **/
@RestController
@RequestMapping("/eb")
public class EBaseController {
    @Resource
    private EBaseService eBaseService;

    /**
     * 发起考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 10:15
     **/
    @PostMapping("/add")
    public ResultVo add(@RequestBody EBase eBase) {
        if (N_Utils.isListEmpty(eBase.getClassids())) return new ResultVo(1, "考试班级不能为空");
        if (N_Utils.isListEmpty(eBase.getSubjectids())) return new ResultVo(1, "考试科目不能为空");
        return eBaseService.save(eBase);
    }

    /**
     * 考试设置
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [subjects]
     * @Author ywx
     * @Date 2022/7/18 10:52
     **/
    @PostMapping("/set_exam")
    public ResultVo setExam(@RequestBody List<ESubject> subjects) {
        for (ESubject s : subjects) {
            try {
                N_Utils.validation(new Object[]{s.getExamid(), "考试id", 1, s.getSubjectid(), "科目id", 2});
            } catch (Exception e) {
                return new ResultVo(1, e.getMessage());
            }
        }
        eBaseService.setExam(subjects);
        return new ResultVo(0, "设置成功");
    }

    /**
     * 考试删除
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 11:30
     **/
    @PostMapping("/delete")
    public ResultVo delete(@RequestBody EBase eBase) {
        eBaseService.deleteById(eBase.getId());
        return new ResultVo(0, "删除成功");
    }

    /**
     * 考试修改
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 11:30
     **/
    @PostMapping("/update")
    public ResultVo update(@RequestBody EBase eBase) {
        return eBaseService.update(eBase);
    }

    @PostMapping("/detail")
    public ResultVo detail(@RequestBody EBase eBase) {
        EBase base = eBaseService.findById(eBase.getId());
        return new ResultVo(0, "获取详情成功", base);
    }

    @PostMapping("/list")
    public ResultVo list(@RequestBody EBase eBase) {
        PageHelper.startPage(eBase.getPage(), eBase.getPageSize());
        List<Map> list = eBaseService.listAll(eBase);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo(list));
        return new ResultVo(0, "获取列表成功", pageResult);
    }

    /**
     * 发布考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 15:21
     **/
    @PostMapping("/fb")
    public ResultVo postExam(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        eBaseService.updateExamState(examid, 1);
        return new ResultVo(0, "发布成功");
    }

    /**
     * 结束考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [eBase]
     * @Author ywx
     * @Date 2022/7/18 15:28
     **/
    @PostMapping("/js")
    public ResultVo endExam(@RequestBody EBase eBase) {
        Integer examid = eBase.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1});
        eBaseService.updateExamState(examid, 2);
        return new ResultVo(0, "结束成功");
    }
}
