package com.xhkjedu.sexam.service.exam;

import com.xhkjedu.sexam.mapper.exam.EBaseMapper;
import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.exam.EClassStudentMapper;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.EPaperMapper;
import com.xhkjedu.sexam.model.exam.EBase;
import com.xhkjedu.sexam.model.exam.EClass;
import com.xhkjedu.sexam.model.exam.EClassStudent;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.EPaper;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class EBaseService {
    @Resource
    private EBaseMapper eBaseMapper;
    @Resource
    private EClassMapper eClassMapper;
    @Resource
    private EClassStudentMapper eClassStudentMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperMapper ePaperMapper;

    /**
     * 发起考试
     *
     * @return com.xhkjedu.vo.ResultVo
     * @Param [model]
     * @Author ywx
     * @Date 2022/7/18 10:37
     **/
    @Transactional(rollbackFor = Exception.class)
    public ResultVo save(EBase model) {
        try {
            //考试
            model.setExamstate(0);
            model.setDeleted(1);
            model.setCreatetime(N_Utils.getSecondTimestamp());
            eBaseMapper.insertUseGeneratedKeys(model);
            Integer examid = model.getId();

            ResultVo vo = saveExam(model, examid);//保存考试基本信息
            if (vo != null) return vo;

            return new ResultVo(0, "保存成功", examid);
        } catch (Exception e) {
            log.error("发起考试失败", e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new ResultVo(1, "保存失败");
        }
    }

    //保存考试基本信息
    private ResultVo saveExam(EBase model, Integer examid) {
        List<Integer> classids = model.getClassids();
        List<EClassStudent> students = eClassStudentMapper.listByClassIds(classids);
        int stuNum = students.stream().distinct().collect(Collectors.toList()).size();
        if (stuNum < students.size()) return new ResultVo(1, "有重复学生");

        //考试班级
        List<EClass> classes = eClassMapper.listByClassIds(classids);
        classes.forEach(c -> c.setExamid(examid));
        eClassMapper.insertList(classes);

        //考试学生
        students.forEach(s -> s.setExamid(examid));
        eClassStudentMapper.insertList(students);

        //考试科目、试卷
        List<String> subjectids = model.getSubjectids();
        List<ESubject> subjects = new ArrayList<>();//考试科目
        List<EPaper> papers = new ArrayList<>();//考试试卷
        String examdate = model.getExamdate();//考试时间
        for (String subjectid : subjectids) {
            ESubject s = new ESubject();
            s.setExamid(examid);
            s.setSubjectid(subjectid);
            s.setSdate(examdate);
            s.setEsstate(0);
            subjects.add(s);
            EPaper p = new EPaper();
            p.setExamid(examid);
            p.setSubjectid(subjectid);
            papers.add(p);
        }
        eSubjectMapper.insertList(subjects);//考试科目

        Integer createid = model.getCreateid();
        Integer createtime = model.getCreatetime();
        for (int i = 0; i < papers.size(); i++) {
            EPaper ePaper = papers.get(i);
            ePaper.setEsid(subjects.get(i).getId());
            ePaper.setCreateid(createid);
            ePaper.setCreatetime(createtime);
        }
        ePaperMapper.insertList(papers);//考试试卷

        return null;
    }

    //删除
    public Integer deleteById(Integer id) {
        return eBaseMapper.deleteById(id);
    }

    //修改
    @Transactional(rollbackFor = Exception.class)
    public ResultVo update(EBase model) {
        try {
            eBaseMapper.updateByPrimaryKeySelective(model);
            Integer examid = model.getExamid();
            eBaseMapper.deleteClassAndSubjectById(examid);//删除关联的班级和科目
            ResultVo vo = saveExam(model, examid);//保存考试基本信息
            if (vo != null) return vo;
            return new ResultVo(0, "修改成功");
        } catch (Exception e) {
            log.error("修改考试失败", e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return new ResultVo(1, "修改失败");
        }
    }

    //获取详情
    public EBase findById(Integer id) {
        return eBaseMapper.selectByPrimaryKey(id);
    }

    //获取列表
    public List<Map> listAll(EBase base) {
        return eBaseMapper.listAll(base);
    }

    //考试设置
    public void setExam(List<ESubject> subjects) {
        eSubjectMapper.setExam(subjects);
    }

    public void updateExamState(Integer examid, int examstate) {
        eBaseMapper.updateExamState(examid, examstate);
    }
}
