package com.xhkjedu.sexam.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.*;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.sexam.vo.paper.EPaperQsVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 附件试卷
 * @Author WN
 * Date 2022/7/18 14:05
 **/
@Service
public class EPaperFjtypeService {
    @Resource
    private EPaperFjtypeMapper ePaperFjtypeMapper;
    @Resource
    private EPaperFjtypeQuestionMapper ePaperFjtypeQuestionMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperFileMapper ePaperFileMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;

    //保存试卷
    @Transactional(rollbackFor = Exception.class)
    public void saveFjPaper(EPaper paper){
        int epid = 0;

        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());

        paper.setPtype(2);//试卷类型1题库2答题卡3仅答题卡
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        int timestamp = N_Utils.getSecondTimestamp();

        //答题卡种题型和试题
        List<EPaperFjtype> fjtypelist = paper.getQuestionStr();
        //答题卡中所有试题
        List<EPaperFjtypeQuestion> queslist = new ArrayList<>();
        fjtypelist.forEach(q->{
            queslist.addAll(q.getQuestions());
        });
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        paper.setAnswered(0);
        ePaperMapper.insertUseGeneratedKeys(paper);
        epid = paper.getId();
        paper.setEpid(epid);

        //保存作业附件
        savePaperFile(paper);
        //保存答题卡信息
        saveFjtypeQuestion(paper,fjtypelist);
        //更加试卷状态是否需要生成分析和更改科目状态
        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，则生成试卷分析，更改科目状态
            savePaperAnalyze(paper,fjtypelist,queslist);
            eSubject.setEsstate(1);
            eSubjectMapper.updateExamSubjectState(eSubject);
        }
    }

    //保存附件试卷文件
    private void savePaperFile(EPaper paper){
        int fileorder = 1;
        List<EPaperFile> files = paper.getPfiles();
        for(EPaperFile f : files){
            f.setCreateid(paper.getCreateid());
            f.setCreatetime(paper.getCreatetime());
            f.setEpid(paper.getEpid());
            f.setFileorder(fileorder);
            fileorder++;
        }
        ePaperFileMapper.insertList(files);
    }

    //保存题型--题型下试题
    private void saveFjtypeQuestion(EPaper paper,List<EPaperFjtype> fjtypelist){
        int qorder = 1;//答题卡试题排序
        int qtorder = 1;//答题卡题型排序

        List<EPaperFjtypeQuestion> saveques = new ArrayList<>();
        for(EPaperFjtype fjt : fjtypelist){
            List<EPaperFjtypeQuestion> quelist = fjt.getQuestions();//题型下试题
            fjt.setEpfjid(paper.getEpid());
            fjt.setCreateid(paper.getHandleid());
            fjt.setCreatetime(paper.getHandletime());
            fjt.setEpfjorder(qtorder);
            fjt.setEpfjnum(quelist.size());
            double tscore = quelist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
            fjt.setEpfjscore(tscore);
             qtorder ++;
        }
        //批量保存题型信息
        ePaperFjtypeMapper.insertList(fjtypelist);
        for(EPaperFjtype fjt : fjtypelist){
            for(EPaperFjtypeQuestion q : fjt.getQuestions()){
                q.setEpfjid(fjt.getId());
                q.setEpid(paper.getEpid());
                q.setOrder(qorder);
                qorder ++;
                saveques.add(q);
            }
        }
        ePaperFjtypeQuestionMapper.insertList(saveques);
    }

    //保存试卷分析
    public void savePaperAnalyze(EPaper ePaper,List<EPaperFjtype> fjtypelist,List<EPaperFjtypeQuestion> queslist){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        for(EPaperFjtypeQuestion q : queslist){
            if(q.getCtype() == 3 || q.getCtype() == 11){
                znum ++;
                zscore = zscore + q.getScore();
            }else{
                knum ++;
                kscore = kscore + q.getScore();
            }
        }

        double znumrate = N_Utils.getIntegerDivideAndMulitiply(znum, ePaper.getPnum());
        double knumrate = 100-znumrate;
        double zscorerate = N_Utils.getDoubleDivideAndMulitiply(zscore, ePaper.getPscore());
        double kscorerate = 100 - zscorerate;

        List<EPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new EPaperQsVo("主观题",zscore,znum,zscorerate,znumrate));
        ranglist.add(new EPaperQsVo("客观题",kscore,knum,kscorerate,knumrate));

        Map ztmap = new HashMap<>();
        ztmap.put("pnum", ePaper.getPnum());
        ztmap.put("pscore",ePaper.getPscore());
        ztmap.put("ranglist",ranglist);

        //题型分布
        List<EPaperQsVo> qtlist = new ArrayList<>();
        for(EPaperFjtype fjt : fjtypelist){
            EPaperQsVo qvo = new EPaperQsVo();
            qvo.setQtname(fjt.getEpfjname());
            qvo.setNum(fjt.getEpfjnum());
            qvo.setScore(fjt.getEpfjscore());
            double scorerate = N_Utils.getDoubleDivideAndMulitiply(fjt.getEpfjscore(), ePaper.getPscore());
            qvo.setSrate(scorerate);
            qtlist.add(qvo);
        }

        EPaperAnalyze epa = new EPaperAnalyze();
        epa.setEpaid(ePaper.getEpid());
        epa.setAlljson(JSON.toJSONString(ztmap));
        epa.setQnumjson(JSON.toJSONString(qtlist));
        ePaperAnalyzeMapper.insertUseGeneratedKeys(epa);
    }

    /**
     * @Description 试卷详情
     * @Param [epid]
     * @Return java.util.Map
     * @Author wn
     * @Date 2022/7/19 8:35
     **/
    public Map getPaperFjDetail(Integer epid){
        Map map = ePaperMapper.getExamPaperDetailByEpid(epid);//试卷信息
        List<Map> fjtypelist = ePaperFjtypeMapper.listPaperFjtypeQuestions(epid);//试卷题型试题信息
        map.put("qtypes",fjtypelist);
        return map;
    }
}
